package logic.module3;

import java.io.File;
import java.io.IOException;
import java.sql.PreparedStatement;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import logic.StopWatch;
import logic.module1.NonFuncCharRemoval;
import logic.module2.CustomAnalyzer;
import logic.module4.Evaluation;

import org.apache.lucene.analysis.KeywordAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Fieldable;
import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.Term;
import org.apache.lucene.index.TermDocs;
import org.apache.lucene.index.TermEnum;
import org.apache.lucene.index.TermPositionVector;
import org.apache.lucene.index.TermVectorOffsetInfo;
import org.apache.lucene.index.IndexReader.FieldOption;

import org.apache.lucene.queryParser.MultiFieldQueryParser;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.ScoreDoc;
import org.apache.lucene.search.Searcher;
import org.apache.lucene.search.TopDocs;
import org.apache.lucene.search.TopScoreDocCollector;
import org.apache.lucene.store.FSDirectory;
import org.apache.lucene.util.Version;

public class TermCategorizer {
	
	private static final String[] KEYWORDARRAY = {"Activate", "Association", "Attach", "Break Bond", "Cause", 
													"Contain", "Create Bond", "Generate", "Inactivate", "Modify",
													"Process", "React", "Release", "Signal", "Substitute"};
	private static final String[] fullFieldsArray = {"RecFullProteinName", "AltFullProteinName", "RecShortName", "AltShortName"};
	private static final String[] partOfFieldsArray = {"RecPartOfProteinName", "RecPartOfShortName", "AltPartOfProteinName", "AltPartOfShortName", };
	
	private IndexReader iReaderMedline;
	private File medlineDir, uniProtDir, keyWordDir;
	private Searcher iSearcherUniprot, iSearcherKeywords;	
	private TermDocs docs;
	private Document doc;	
	private ArrayList<String> pmidList;
	private Map<String, ArrayList<Entity>> pmidToKeywords;
	private Map<String, ArrayList<Entity>> pmidToProteins;
	private ArrayList<Entity> keywordsLocated;
	private ArrayList<Entity> proteinsLocated;
	private ArrayList<Entity> potensialProteinsLocated;
	private StopWatch sw;
	private int docNumber;
	private NonFuncCharRemoval nonFuncCharRemoval;
	private TermPositionVector tpv;
	private Entity prev = null;
	
	public TermCategorizer() {
		
		sw = new StopWatch();
		
		medlineDir = new File("MEDLINE_index");
		uniProtDir = new File("UniProt_index");
		keyWordDir = new File("Keyword_index");
		
		pmidToKeywords = new TreeMap<String, ArrayList<Entity>>();
		pmidToProteins = new TreeMap<String, ArrayList<Entity>>();
		pmidList = new ArrayList<String>();
		nonFuncCharRemoval = new NonFuncCharRemoval();
		
		if (medlineDir.canRead() && uniProtDir.canRead() && keyWordDir.canRead()) {
			try {
				sw.start();
				iReaderMedline = IndexReader.open(FSDirectory.open(medlineDir));
				iSearcherUniprot = new IndexSearcher(FSDirectory.open(uniProtDir), true);
				iSearcherKeywords = new IndexSearcher(FSDirectory.open(keyWordDir), true);
														
				System.out.println("Starting Term Categorizing....");
				docs = iReaderMedline.termDocs();
				
				while(docs.next()) {
					docNumber = docs.doc();
					doc = iReaderMedline.document(docNumber);
					List<Fieldable> fields= doc.getFields();
					String pmid = "";

					keywordsLocated = new ArrayList<Entity>();
					proteinsLocated = new ArrayList<Entity>();
					potensialProteinsLocated = new ArrayList<Entity>();
										
					for(Fieldable f : fields) {
						String value = f.stringValue();
						if("PMID".equals(f.name())) {
							pmid = value;
							pmidList.add(pmid);
						} else {
							tpv = (TermPositionVector)iReaderMedline.getTermFreqVector(docNumber, f.name());
							String[] tokens = nonFuncCharRemoval.preProcessInit(value).trim().split("[\\s]+");
							
							this.checkForPotensialProtein(tokens, f.name());							
							this.checkForPPIwords(tokens, f.name());							
						}
					}
					
					Collections.sort(potensialProteinsLocated);
					for(Entity e : potensialProteinsLocated) {
						checkForFullName(e);
						this.proteinNameCombination(e);
					}
					
					if(proteinsLocated.size() > 0) {
						Collections.sort(proteinsLocated);
						pmidToProteins.put(pmid, proteinsLocated);						
					} else if(proteinsLocated.size() == 0) {
						Collections.sort(potensialProteinsLocated);
						pmidToProteins.put(pmid, potensialProteinsLocated);
					}
					
					if(keywordsLocated.size() > 0) {
						Collections.sort(keywordsLocated);
						pmidToKeywords.put(pmid, keywordsLocated);
					}
					
					System.out.println("Finished with doc #"+docNumber);
				}
				
				iSearcherUniprot.close();
				iSearcherKeywords.close();
				iReaderMedline.close();
				
				sw.stop();
				System.out.println("TermCategorising used  " + sw.getElapsedTime() + "milliseconds to complete");
				
				new Evaluation(pmidToProteins, pmidToKeywords, pmidList);				
			} catch (CorruptIndexException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			} 
		} else { System.out.println("Cant read index directory");}	
	}
	
	private void checkForFullName(Entity e) {
		TopScoreDocCollector collector;
		Query q = null;
		String fullName = PreQueryProcess.escapeForFullQuery(e.getElement()).toLowerCase().trim();
		
		if(PreQueryProcess.useableTokenCheck(fullName)) {
			collector = TopScoreDocCollector.create(5, true);
			try {
				q = new MultiFieldQueryParser(Version.LUCENE_30, fullFieldsArray, CustomAnalyzer.getInstance()).parse(fullName);
				iSearcherUniprot.search(q, collector);
				tpv = (TermPositionVector)iReaderMedline.getTermFreqVector(docNumber, e.getSection());
			} catch (ParseException exp) {
				exp.printStackTrace();
			} catch (IOException exp) {
				exp.printStackTrace();
			}
			
			TermVectorOffsetInfo[] offsets = tpv.getOffsets(tpv.indexOf(fullName));
			
			for (TermVectorOffsetInfo t : offsets) {
				Entity proteinEntity = new Entity(e.getSection(), t.getStartOffset()+"-"+t.getEndOffset(), fullName);
				if (collector.getTotalHits() > 0 && !proteinsLocated.contains(proteinEntity)) {
					proteinsLocated.add(proteinEntity);
				} else {
					continue;
				}
			}
		}
	}
	
	private void checkForPotensialProtein(String[] tokens, String field) {
		TopScoreDocCollector collector;
		Query q = null;
		
		for(String s : tokens) {
			String partName = PreQueryProcess.partOfNamePreProcess(s).toLowerCase().trim();
			String [] secondaryTokens = partName.trim().split("[\\s]+");
			
			for(String innerS : secondaryTokens) {
				if(PreQueryProcess.useableTokenCheck(innerS)) {
					collector = TopScoreDocCollector.create(5, true);
					try {
						q = new MultiFieldQueryParser(Version.LUCENE_30, partOfFieldsArray, CustomAnalyzer.getInstance()).parse(innerS);
						iSearcherUniprot.search(q, collector);
						tpv = (TermPositionVector)iReaderMedline.getTermFreqVector(docNumber, field);

					} catch (ParseException e) {
						e.printStackTrace();
					} catch (IOException e) {
						e.printStackTrace();
					}
					
					TermVectorOffsetInfo[] offsets = tpv.getOffsets(tpv.indexOf(innerS));
					
					for (TermVectorOffsetInfo t : offsets) {
						Entity e = new Entity(field, t.getStartOffset()+"-"+t.getEndOffset(), innerS);
						if (collector.getTotalHits() > 0 && !potensialProteinsLocated.contains(e)) {
							potensialProteinsLocated.add(e);
						} else {
							continue;
						}
					}
				}
			}
		}		
	}
	
	private void checkForPPIwords(String[] tokens, String field) {
		
		TopScoreDocCollector collector;
		Query q = null;
		
		for(String s : tokens) {
			s = PreQueryProcess.partOfNamePreProcess(s).toLowerCase().trim();
			String[] secondaryTokens = s.trim().split("[\\s]+");
			for(String innerS : secondaryTokens) {
				if(PreQueryProcess.useableTokenCheck(innerS)) {
					collector = TopScoreDocCollector.create(1, true);
					try {
						q = new MultiFieldQueryParser(Version.LUCENE_30, KEYWORDARRAY, CustomAnalyzer.getInstance()).parse(innerS);
						iSearcherKeywords.search(q, collector);
						tpv = (TermPositionVector)iReaderMedline.getTermFreqVector(docNumber, field);
					} catch (ParseException e) {
						e.printStackTrace();
					} catch (IOException e) {
						e.printStackTrace();
					}
					
					TermVectorOffsetInfo[] offsets = tpv.getOffsets(tpv.indexOf(innerS));
					
					for (TermVectorOffsetInfo t : offsets) {
						Entity e = new Entity(field, t.getStartOffset()+"-"+t.getEndOffset(), innerS);
						if (collector.getTotalHits() > 0 && !keywordsLocated.contains(e)) {
							keywordsLocated.add(e);
						} else {
							continue;
						}
					}
				}
			}
		}
	}
	
	private void proteinNameCombination(Entity e) {		
		Entity ent = null;
		
		if (prev != null) {
			if (prev.getSection().equals(e.getSection()) && (Integer.parseInt(e.getOffset()[0]) - Integer.parseInt(prev.getOffset()[1])) == 1) {
				ent = new Entity(e.getSection(), prev.getOffset()[0] + "-" + e.getOffset()[1], prev.getElement() + " " + e.getElement());
				prev = ent;
			} else {
				this.checkForFullName(prev);
				prev = e;
			}			
		} else {
			prev = e;
		}		
	}

	public static void main(String[] args) {
		new TermCategorizer();
	}
}