#!/usr/bin/env python

import sys
import re

# Evaluation script for unified PPI corpora. Expects two arguments, the
# first giving the name of the answer key file and the second the name
# of a file with the predicitions of the evaluated program.

if len(sys.argv) != 3:
    print >> sys.stderr, "Usage:", sys.argv[0], "ANSWER-KEY PREDICTIONS"
    sys.exit(1)


def errorExit(s):
    print >> sys.stderr, s
    sys.exit(1)

def openFile(s):
    """Returns file with given name opened for reading, exits
    gracefully on failure."""
    try:
        f = open(s)
    except IOError, e:
        (errno, strerror) = e
        errorExit("Failed to open "+s+": "+strerror)
    return f

def parseAnswers(f):
    """Parses the contents of the open file f, returning a dictionary
    keyed by pair id with answer as values. Exits on failure."""

    d = {}

    for lineNo, line in enumerate(f):
        line = line.rstrip()

        m = re.match(r'\s*(\S+)\s*(\S+)\s*$', line)
        if not m:
            errorExit("Format error on line "+str(lineNo+1)+" of file "+f.name+": Each line should contain only PAIR-ID and LABEL (line '"+line+"')")

        pairId, label = m.groups()

        if label not in ("True", "False"):
            errorExit("Format error on line "+str(lineNo+1)+" of file "+f.name+": label should be either 'True' or 'False' (label '"+label+"')")

        if pairId in d:
            errorExit("Error on line "+str(lineNo+1)+" of file "+f.name+": PAIR-ID '"+pairId+"' seen twice in file.")

        d[pairId] = label

    return d


# Read in correct answers and predictions

answerFile = openFile(sys.argv[1])
answer     = parseAnswers(answerFile)
answerFile.close()

predictionFile = openFile(sys.argv[2])
prediction     = parseAnswers(predictionFile)
predictionFile.close()


# Some sanity checking

if len(answer) == 0:
    errorExit("Error: empty answer file")

if len(answer) != len(prediction):
    errorExit("Error: number of answers ("+str(len(answer))+") doesn't match number of predictions ("+str(len(prediction))+")")

missing = [k for k in answer if k not in prediction]
if len(missing) != 0:
    errorExit("Error: missing predictions for the following pair ids:\n"+", ".join(missing))

# (this should never happen actually)
extra = [k for k in prediction if k not in answer]
if len(extra) != 0:
    errorExit("Error: extra predictions for the following pair ids not in answer:\n"+", ".join(extra))

assert sorted(answer.keys()) == sorted(prediction.keys()), "INTERNAL ERROR"


# keys match, calculate true positive etc.

tp = fp = tn = fn = 0
for k in answer:
    if prediction[k] == "True":
        if prediction[k] == answer[k]:
            tp += 1
        else:
            fp += 1
    elif prediction[k] == "False":
        if prediction[k] == answer[k]:
            tn += 1
        else:
            fn += 1
    else:
        errorExit("INTERNAL ERROR")
    

# sanity
assert tp + fp + tn + fn == len(answer), "INTERNAL ERROR"


# calculate and print out stats

acc = float(tp+tn) / (tp+fp+tn+fn)

if tp+fp != 0:
    prec = float(tp) / (tp+fp)
else:
    prec = 0.0

if tp + fn != 0:
    rec = float(tp) / (tp+fn)
else:
    rec = 0.0

if prec+rec != 0.0:
    f = 2*prec*rec / (prec+rec)
else:
    f = 0.0

print "Evaluation:", sys.argv[1], "vs.", sys.argv[2]
print "total number of pairs %d" % (tp+fp+tn+fn)
print "true positive %d, false positive %d, true negative %d, false negative %d" % (tp, fp, tn, fn)
print "precision %.3f, recall %.3f, F %.3f, accuracy %.3f" % (prec, rec, f, acc)

