package testing.yapex;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import logic.StopWatch;
import logic.module1.NonFuncCharRemoval;
import logic.module2.CustomAnalyzer;
import logic.module3.Entity;
import logic.module3.PreQueryProcess;
import logic.module4.Output;

import org.apache.lucene.document.Document;
import org.apache.lucene.document.Fieldable;
import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.TermDocs;
import org.apache.lucene.index.TermPositionVector;
import org.apache.lucene.index.TermVectorOffsetInfo;
import org.apache.lucene.queryParser.MultiFieldQueryParser;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.Searcher;
import org.apache.lucene.search.TopScoreDocCollector;
import org.apache.lucene.store.FSDirectory;
import org.apache.lucene.util.Version;

public class YapexTermCategorizer {
	
	private static final String[] fullFieldsArray = {"RecFullProteinName", "AltFullProteinName", "RecShortName", "AltShortName"};
	private static final String[] partOfFieldsArray = {"RecPartOfProteinName", "RecPartOfShortName", "AltPartOfProteinName", "AltPartOfShortName", };
	
	private IndexReader iReaderMedline;
	private File medlineDir, uniProtDir, keyWordDir;
	private Searcher iSearcherUniprot;
	private TermDocs docs;
	private Document doc;	
	private ArrayList<Integer> pmidList;
	private Map<String, ArrayList<Entity>> pmidToProteins;
	private ArrayList<Entity> proteinsLocated;
	private ArrayList<Entity> potensialProteinsLocated;
	private ArrayList<Entity> backupProteinList;
	private int docNumber;
	private NonFuncCharRemoval nonFuncCharRemoval;
	private TermPositionVector tpv;
	private Entity prev = null;
	private boolean lastChance;
	private Output out;
	
	public YapexTermCategorizer() {
		medlineDir = new File("MEDLINE_index");
		uniProtDir = new File("UniProt_index");
		
		pmidToProteins = new TreeMap<String, ArrayList<Entity>>();
		pmidList = new ArrayList<Integer>();
		nonFuncCharRemoval = new NonFuncCharRemoval();
		lastChance = false;
		out = new Output("yapex_results.txt");
		
		if (medlineDir.canRead() && uniProtDir.canRead()) {
			
			try {
				iReaderMedline = IndexReader.open(FSDirectory.open(medlineDir));
				iSearcherUniprot = new IndexSearcher(FSDirectory.open(uniProtDir), true);
				docs = iReaderMedline.termDocs();
				
				while(docs.next()) {
					docNumber = docs.doc();
					doc = iReaderMedline.document(docNumber);
					List<Fieldable> fields= doc.getFields();
					String pmid = "";
					
					proteinsLocated = new ArrayList<Entity>();
					potensialProteinsLocated = new ArrayList<Entity>();
					backupProteinList = new ArrayList<Entity>();
										
					for(Fieldable f : fields) {
						String value = f.stringValue();
						if("PMID".equals(f.name())) {
							pmid = value;
							pmidList.add(Integer.parseInt(pmid));
						} else {
							tpv = (TermPositionVector)iReaderMedline.getTermFreqVector(docNumber, f.name());
							String[] tokens = nonFuncCharRemoval.preProcessInit(value).trim().split("[\\s]+");
							
							this.checkForPotensialProtein(tokens, f.name());
						}
					}
					
					Collections.sort(potensialProteinsLocated);
					for(Entity e : potensialProteinsLocated) {
						lastChance = false;
						checkForFullName(e);
						lastChance = true;
						this.proteinNameCombination(e);
					}
					
					if(proteinsLocated.size() > 0) {
						Collections.sort(proteinsLocated);
						pmidToProteins.put(pmid, proteinsLocated);
					
					} else if(proteinsLocated.size() == 0) {
						Collections.sort(backupProteinList);
						pmidToProteins.put(pmid, backupProteinList);
					}
					
					System.out.println("Finished with doc #"+docNumber);
				}				
				
				iReaderMedline.close();
				iSearcherUniprot.close();
							
				Collections.sort(pmidList);
				
				for(int s : pmidList) {
					String key = Integer.valueOf(s).toString();
					for(Entity e : pmidToProteins.get(key)){
						out.write(s + ":" + e.getSection() + ":" + e.getElement());
					}				
				}
				
				out.close();
			} catch (CorruptIndexException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}			
		}
	}
		
	private void checkForFullName(Entity e) {
		TopScoreDocCollector collector;
		Query q = null;
		String fullName = PreQueryProcess.escapeForFullQuery(e.getElement()).toLowerCase().trim();

		if(PreQueryProcess.useableTokenCheck(fullName)) {
			collector = TopScoreDocCollector.create(5, true);
			try {
				q = new MultiFieldQueryParser(Version.LUCENE_30, fullFieldsArray, CustomAnalyzer.getInstance()).parse(fullName);
				iSearcherUniprot.search(q, collector);
				tpv = (TermPositionVector)iReaderMedline.getTermFreqVector(docNumber, e.getSection());
			} catch (ParseException exp) {
				exp.printStackTrace();
			} catch (IOException exp) {
				exp.printStackTrace();
			}
			
			int i = tpv.indexOf(fullName);
			
			if(i == -1 && !backupProteinList.contains(e) && lastChance) {
				backupProteinList.add(e);
			} else {
				TermVectorOffsetInfo[] offsets = tpv.getOffsets(i);
				for (TermVectorOffsetInfo t : offsets) {
					Entity proteinEntity = new Entity(e.getSection(), t.getStartOffset()+"-"+t.getEndOffset(), fullName);
					if (collector.getTotalHits() > 0 && !proteinsLocated.contains(proteinEntity)) {
						proteinsLocated.add(proteinEntity);
					} else {
						continue;
					}
				}
			}			
		}
	}

	private void checkForPotensialProtein(String[] tokens, String field) {
		TopScoreDocCollector collector;
		Query q = null;

		for(String s : tokens) {
			String partName = PreQueryProcess.partOfNamePreProcess(s).toLowerCase().trim();
			String [] secondaryTokens = partName.trim().split("[\\s]+");

			for(String innerS : secondaryTokens) {
				if(PreQueryProcess.useableTokenCheck(innerS)) {
					collector = TopScoreDocCollector.create(5, true);
					try {
						q = new MultiFieldQueryParser(Version.LUCENE_30, partOfFieldsArray, CustomAnalyzer.getInstance()).parse(innerS);
						iSearcherUniprot.search(q, collector);
						tpv = (TermPositionVector)iReaderMedline.getTermFreqVector(docNumber, field);

					} catch (ParseException e) {
						e.printStackTrace();
					} catch (IOException e) {
						e.printStackTrace();
					}

					TermVectorOffsetInfo[] offsets = tpv.getOffsets(tpv.indexOf(innerS));

					for (TermVectorOffsetInfo t : offsets) {
						Entity e = new Entity(field, t.getStartOffset()+"-"+t.getEndOffset(), innerS);
						if (collector.getTotalHits() > 0 && !potensialProteinsLocated.contains(e)) {
							potensialProteinsLocated.add(e);
						} else {
							continue;
						}
					}
				}
			}
		}		
	}
	
	private void proteinNameCombination(Entity e) {		
		Entity ent = null;
		
		if (prev != null) {
			if (prev.getSection().equals(e.getSection()) && (Integer.parseInt(e.getOffset()[0]) - Integer.parseInt(prev.getOffset()[1])) == 1) {
				ent = new Entity(e.getSection(), prev.getOffset()[0] + "-" + e.getOffset()[1], prev.getElement() + " " + e.getElement());
				prev = ent;
			} else {
				this.checkForFullName(prev);
				prev = e;
			}			
		} else {
			prev = e;
		}		
	}
	
	public static void main(String[] args) {
		new YapexTermCategorizer();
	}
}
